//Use command 'g++ -std=gnu++14 stabilityRegion_plot.cpp -lmgl' to compile this plotting script
#include <cmath>
#include <vector>
#include <complex>
#include <iostream>
#include <eigen3/Eigen/Dense>
#include <mgl2/mgl.h>
using namespace Eigen;

////TASK C
// This function computes |S(z)| for a chosen value of 'gamma'
double computeSMag(std::complex<double> z, double gamma){
    double Smag;
    std::complex<double> Stab_func;
    
    //To-do [task (c)]: Write stability function in terms of 'z' and gamma
    Stab_func = (1.0+z*(1.0-2.0*gamma)+z*z*(0.5-2.0*gamma+pow(gamma,2)))/(pow((1.0-z*gamma),2));
    
    Smag = std::abs(Stab_func);
    
    return Smag;
}
////END OF TASK C

int main() {
    //To-do [task (c)]: : Modify value for gamma in next line
    double gamma = (3.0-sqrt(3.0))/6.0;
    //double gamma = (3.0+sqrt(3.0))/6.0;
    //double gamma = 1.0;

    const std::complex<double> I(0.0, 1.0); //complex number 'iota'=sqrt(-1)
    double x_min = -20.0;
    double x_max = 20.0;
    double y_min = -20.0;
    double y_max = 20.0;
    int N_x = 1001;
    int N_y = 1001;
    double delta_x = (x_max - x_min)/(N_x-1);
    double delta_y = (y_max - y_min)/(N_y-1);
    VectorXd x_grid = VectorXd::LinSpaced(N_x, x_min, x_max);
    VectorXd y_grid = VectorXd::LinSpaced(N_y, y_min, y_max);
    
    // generate stability curve    
    MatrixXcd z_grid(N_x,N_y);
    MatrixXd Smag_grid(N_x,N_y);
    MatrixXd Smag_grid_boolean(N_x,N_y);
    Smag_grid_boolean.setOnes();
    for(int k=0; k < N_y; ++k){
        for(int j=0; j < N_x; ++j){
            z_grid(j,k) = x_grid(j)+I*y_grid(k); // z = x + iota * y;
            Smag_grid(j,k) = computeSMag(z_grid(j,k),gamma); // Compute |S(z)| for given 'gamma'
            if (Smag_grid(j,k) >= 1.0) Smag_grid_boolean(j,k) = 0.0; //Create a boolean variable that takes value = 0.0 wherever |S(z)| >= 1 and takes value = 1.0 wherever |S(z)| < 1.
        }
    }
    
    // Plot stability function using MathGL (Blue region indicates region of |S(z)| < 1 )
    mglData x_dat(x_grid.data(), x_grid.size());
    mglData y_dat(y_grid.data(), y_grid.size());
    mglData dat(Smag_grid_boolean.rows(), Smag_grid_boolean.cols(), Smag_grid_boolean.data());
    mglGraph gr;
    gr.Title("Blue Region: |S(z)| < 1");
    gr.SetRanges(x_min,x_max,y_min,y_max);
    gr.Grid("xy","g");
    gr.Axis(); gr.Label('x',"Real(z)",0); gr.Label('y',"Im(z)",0);
    gr.Alpha(true);
    gr.SetRange('c',0.0,1.0);
    gr.Tile(x_dat,y_dat,dat,"wb");
    gr.WriteFrame("stabilityPlot1.png");
    //gr.WriteFrame("stabilityPlot2.png");

    return 0;
}


// End of file
